#!/bin/sh

# download and patch quicklisp to use curl
# Copyright (C) 2022  cage

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.
# If not, see [[http://www.gnu.org/licenses/][http://www.gnu.org/licenses/]].

# Important note: this works only if curl is installed (obviously) and
# with SBCL

QUICKLISP_INSTALL_DIR=$HOME/quicklisp/

QUICKLISP_INSTALL_CLIENT_DIR=$QUICKLISP_INSTALL_DIR/quicklisp/

QUICKLISP_URL="https://beta.quicklisp.org/quicklisp.lisp"
QUICKLISP_SIG_URL="https://beta.quicklisp.org/quicklisp.lisp.asc"
QUICKLISP_KEY_URL="https://beta.quicklisp.org/release-key.txt"

QUICKLISP="quicklisp.lisp"
QUICKLISP_SIG="quicklisp.lisp.asc"
QUICKLISP_KEY="release-key"

QUICKLISP_SIGNATURE="D7A3 +489D +DEFE +32B7 +D0E7 +CC61 +3079 +65AB +028B +5FF7"

LISP_SOURCE_REGISTRY_DIR="$HOME/.config/common-lisp/"

LISP_SOURCE_REGISTRY_FILE="$LISP_SOURCE_REGISTRY_DIR/source-registry.conf"

VERIFY_OK_RES=2

echo_bold () {
    printf "\033[1m%s\033[0m\n" "$1"
}

check_quicklisp () {
    if [ -d "$QUICKLISP_INSTALL_DIR" ]; then
        echo 0;
    else
        echo 1;
    fi
}

check_quicklisp_signature () {
    chk1_prog='BEGIN {res=0} /Good signature.*release@quicklisp.org/ {res++; print res}'
    chk2_prog="BEGIN {res=0} /${QUICKLISP_SIGNATURE}/ {res++; print res}"
    res1=$(LC_MESSAGES="C" /usr/bin/gpg --verify quicklisp.lisp.asc quicklisp.lisp 2>&1 | gawk  -- "${chk1_prog}")
    res2=$(/usr/bin/gpg --verify quicklisp.lisp.asc quicklisp.lisp 2>&1 | gawk  -- "${chk2_prog}")
    res=$((res1 + res2))
    printf "%s" "$res"
}

patch_installer () {
    patch quicklisp.lisp < installer.patch
}

patch_quicklisp_client () {
    cp client.patch $QUICKLISP_INSTALL_CLIENT_DIR
    cd $QUICKLISP_INSTALL_CLIENT_DIR && echo "$PWD" &&  patch http.lisp < client.patch
}


install_quicklisp () {
    echo_bold "Downloading quicklisp..."
    /usr/bin/curl "$QUICKLISP_URL"     > $QUICKLISP
    /usr/bin/curl "$QUICKLISP_SIG_URL" > $QUICKLISP_SIG
    /usr/bin/curl "$QUICKLISP_KEY_URL" > $QUICKLISP_KEY
    echo_bold "Importing gpg key."
    /usr/bin/gpg --import $QUICKLISP_KEY
    echo_bold "Verifing key"
    signature_verified=$(check_quicklisp_signature)
    if [ "$signature_verified" -ne $VERIFY_OK_RES ]; then
         echo_bold "Key verification failed!"
         exit 1
    else
        echo "ok"
        echo_bold "Key sucessfully verified."
        echo_bold "patching installer"
        patch_installer
        if test "$1" = "" ; then
            /usr/bin/sbcl --load $QUICKLISP           \
             --eval "(quicklisp-quickstart:install)"    \
             --eval "(ql:add-to-init-file)"             \
             --eval "(sb-ext:quit)";
        else
            /usr/bin/sbcl --load $QUICKLISP                           \
             --eval "(quicklisp-quickstart:install)"                    \
             --eval "(ql-util:without-prompting (ql:add-to-init-file))" \
             --eval "(sb-ext:quit)";
        fi
        /bin/mkdir -p "$LISP_SOURCE_REGISTRY_DIR"
        PAR_PWD=$(/usr/bin/dirname "$PWD")
        echo "(:source-registry"       >  "$LISP_SOURCE_REGISTRY_FILE"
        echo "  (:tree \"$PAR_PWD\")"  >> "$LISP_SOURCE_REGISTRY_FILE"
        echo ":inherit-configuration)" >> "$LISP_SOURCE_REGISTRY_FILE"
        echo_bold "quicklisp installed"
        echo_bold "patching client"
        patch_quicklisp_client
    fi
}

quicklisp_installed_p=$(check_quicklisp)

if [ $quicklisp_installed_p -eq 0 ]; then
    echo_bold "Quicklisp already installed."
else
    if test "$1" = "--do-not-prompt" ; then
        install_quicklisp 1
    else
        install_quicklisp
    fi
fi

echo_bold "Finished."
